# SPDX-License-Identifier: (Apache-2.0 OR MIT)

import pytest

import orjson


@pytest.mark.parametrize(
    "input",
    [
        b'"\xc8\x93',
        b'"\xc8',
    ],
)
def test_invalid(input):
    with pytest.raises(orjson.JSONDecodeError):
        orjson.loads(input)



# SPDX-License-Identifier: (Apache-2.0 OR MIT)

import lzma
import os
from pathlib import Path
from typing import Any, Dict

import orjson

dirname = os.path.join(os.path.dirname(__file__), "../data")

STR_CACHE: Dict[str, str] = {}

OBJ_CACHE: Dict[str, Any] = {}


def read_fixture_bytes(filename, subdir=None):
    if subdir is None:
        path = Path(dirname, filename)
    else:
        path = Path(dirname, subdir, filename)
    if path.suffix == ".xz":
        contents = lzma.decompress(path.read_bytes())
    else:
        contents = path.read_bytes()
    return contents


def read_fixture_str(filename, subdir=None):
    if filename not in STR_CACHE:
        STR_CACHE[filename] = read_fixture_bytes(filename, subdir).decode("utf-8")
    return STR_CACHE[filename]


def read_fixture_obj(filename):
    if filename not in OBJ_CACHE:
        OBJ_CACHE[filename] = orjson.loads(read_fixture_str(filename))
    return OBJ_CACHE[filename]



# SPDX-License-Identifier: (Apache-2.0 OR MIT)

import pytest

import orjson


class TestDict:
    def test_dict(self):
        """
        dict
        """
        obj = {"key": "value"}
        ref = '{"key":"value"}'
        assert orjson.dumps(obj) == ref.encode("utf-8")
        assert orjson.loads(ref) == obj

    def test_dict_duplicate_loads(self):
        assert orjson.loads(b'{"1":true,"1":false}') == {"1": False}

    def test_dict_empty(self):
        obj = [{"key": [{}] * 4096}] * 4096  # type:ignore
        assert orjson.loads(orjson.dumps(obj)) == obj

    def test_dict_large_dict(self):
        """
        dict with >512 keys
        """
        obj = {"key_%s" % idx: [{}, {"a": [{}, {}, {}]}, {}] for idx in range(513)}  # type: ignore
        assert len(obj) == 513
        assert orjson.loads(orjson.dumps(obj)) == obj

    def test_dict_large_4096(self):
        """
        dict with >4096 keys
        """
        obj = {"key_%s" % idx: "value_%s" % idx for idx in range(4097)}
        assert len(obj) == 4097
        assert orjson.loads(orjson.dumps(obj)) == obj

    def test_dict_large_65536(self):
        """
        dict with >65536 keys
        """
        obj = {"key_%s" % idx: "value_%s" % idx for idx in range(65537)}
        assert len(obj) == 65537
        assert orjson.loads(orjson.dumps(obj)) == obj

    def test_dict_large_keys(self):
        """
        dict with keys too large to cache
        """
        obj = {
            "keeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeey": "value"
        }
        ref = '{"keeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeey":"value"}'
        assert orjson.dumps(obj) == ref.encode("utf-8")
        assert orjson.loads(ref) == obj

    def test_dict_unicode(self):
        """
        dict unicode keys
        """
        obj = {"🐈": "value"}
        ref = b'{"\xf0\x9f\x90\x88":"value"}'
        assert orjson.dumps(obj) == ref
        assert orjson.loads(ref) == obj
        assert orjson.loads(ref)["🐈"] == "value"

    def test_dict_invalid_key_dumps(self):
        """
        dict invalid key dumps()
        """
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps({1: "value"})
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps({b"key": "value"})

    def test_dict_invalid_key_loads(self):
        """
        dict invalid key loads()
        """
        with pytest.raises(orjson.JSONDecodeError):
            orjson.loads('{1:"value"}')
        with pytest.raises(orjson.JSONDecodeError):
            orjson.loads('{{"a":true}:true}')

    def test_dict_similar_keys(self):
        """
        loads() similar keys

        This was a regression in 3.4.2 caused by using
        the implementation in wy instead of wyhash.
        """
        assert orjson.loads(
            '{"cf_status_firefox67": "---", "cf_status_firefox57": "verified"}'
        ) == {"cf_status_firefox57": "verified", "cf_status_firefox67": "---"}

    def test_dict_pop_replace_first(self):
        "Test pop and replace a first key in a dict with other keys."
        data = {"id": "any", "other": "any"}
        data.pop("id")
        assert orjson.dumps(data) == b'{"other":"any"}'
        data["id"] = "new"
        assert orjson.dumps(data) == b'{"other":"any","id":"new"}'

    def test_dict_pop_replace_last(self):
        "Test pop and replace a last key in a dict with other keys."
        data = {"other": "any", "id": "any"}
        data.pop("id")
        assert orjson.dumps(data) == b'{"other":"any"}'
        data["id"] = "new"
        assert orjson.dumps(data) == b'{"other":"any","id":"new"}'

    def test_dict_pop(self):
        "Test pop and replace a key in a dict with no other keys."
        data = {"id": "any"}
        data.pop("id")
        assert orjson.dumps(data) == b"{}"
        data["id"] = "new"
        assert orjson.dumps(data) == b'{"id":"new"}'

    def test_in_place(self):
        "Mutate dict in-place"
        data = {"id": "any", "static": "msg"}
        data["id"] = "new"
        assert orjson.dumps(data) == b'{"id":"new","static":"msg"}'

    def test_dict_0xff(self):
        "dk_size <= 0xff"
        data = {str(idx): idx for idx in range(0, 0xFF)}
        data.pop("112")
        data["112"] = 1
        data["113"] = 2
        assert orjson.loads(orjson.dumps(data)) == data

    def test_dict_0xff_repeated(self):
        "dk_size <= 0xff repeated"
        for _ in range(0, 100):
            data = {str(idx): idx for idx in range(0, 0xFF)}
            data.pop("112")
            data["112"] = 1
            data["113"] = 2
            assert orjson.loads(orjson.dumps(data)) == data

    def test_dict_0xffff(self):
        "dk_size <= 0xffff"
        data = {str(idx): idx for idx in range(0, 0xFFFF)}
        data.pop("112")
        data["112"] = 1
        data["113"] = 2
        assert orjson.loads(orjson.dumps(data)) == data

    def test_dict_0xffff_repeated(self):
        "dk_size <= 0xffff repeated"
        for _ in range(0, 100):
            data = {str(idx): idx for idx in range(0, 0xFFFF)}
            data.pop("112")
            data["112"] = 1
            data["113"] = 2
            assert orjson.loads(orjson.dumps(data)) == data

    def test_dict_dict(self):
        class C:
            def __init__(self):
                self.a = 0
                self.b = 1

        assert orjson.dumps(C().__dict__) == b'{"a":0,"b":1}'



# SPDX-License-Identifier: (Apache-2.0 OR MIT)

import datetime
import json

import orjson

from .util import read_fixture_obj


class TestIndentedOutput:
    def test_equivalent(self):
        """
        OPT_INDENT_2 is equivalent to indent=2
        """
        obj = {"a": "b", "c": {"d": True}, "e": [1, 2]}
        assert orjson.dumps(obj, option=orjson.OPT_INDENT_2) == json.dumps(
            obj, indent=2
        ).encode("utf-8")

    def test_sort(self):
        obj = {"b": 1, "a": 2}
        assert (
            orjson.dumps(obj, option=orjson.OPT_INDENT_2 | orjson.OPT_SORT_KEYS)
            == b'{\n  "a": 2,\n  "b": 1\n}'
        )

    def test_non_str(self):
        obj = {1: 1, "a": 2}
        assert (
            orjson.dumps(obj, option=orjson.OPT_INDENT_2 | orjson.OPT_NON_STR_KEYS)
            == b'{\n  "1": 1,\n  "a": 2\n}'
        )

    def test_options(self):
        obj = {
            1: 1,
            "b": True,
            "a": datetime.datetime(1970, 1, 1),
        }
        assert (
            orjson.dumps(
                obj,
                option=orjson.OPT_INDENT_2
                | orjson.OPT_SORT_KEYS
                | orjson.OPT_NON_STR_KEYS
                | orjson.OPT_NAIVE_UTC,
            )
            == b'{\n  "1": 1,\n  "a": "1970-01-01T00:00:00+00:00",\n  "b": true\n}'
        )

    def test_empty(self):
        obj = [{}, [[[]]], {"key": []}]
        ref = b'[\n  {},\n  [\n    [\n      []\n    ]\n  ],\n  {\n    "key": []\n  }\n]'
        assert orjson.dumps(obj, option=orjson.OPT_INDENT_2) == ref

    def test_twitter_pretty(self):
        """
        twitter.json pretty
        """
        obj = read_fixture_obj("twitter.json.xz")
        assert orjson.dumps(obj, option=orjson.OPT_INDENT_2) == json.dumps(
            obj, indent=2, ensure_ascii=False
        ).encode("utf-8")

    def test_github_pretty(self):
        """
        github.json pretty
        """
        obj = read_fixture_obj("github.json.xz")
        assert orjson.dumps(obj, option=orjson.OPT_INDENT_2) == json.dumps(
            obj, indent=2, ensure_ascii=False
        ).encode("utf-8")

    def test_canada_pretty(self):
        """
        canada.json pretty
        """
        obj = read_fixture_obj("canada.json.xz")
        assert orjson.dumps(obj, option=orjson.OPT_INDENT_2) == json.dumps(
            obj, indent=2, ensure_ascii=False
        ).encode("utf-8")

    def test_citm_catalog_pretty(self):
        """
        citm_catalog.json pretty
        """
        obj = read_fixture_obj("citm_catalog.json.xz")
        assert orjson.dumps(obj, option=orjson.OPT_INDENT_2) == json.dumps(
            obj, indent=2, ensure_ascii=False
        ).encode("utf-8")



# SPDX-License-Identifier: (Apache-2.0 OR MIT)

from json import dumps as _json_dumps
from json import loads as json_loads

from rapidjson import dumps as _rapidjson_dumps
from rapidjson import loads as rapidjson_loads
from simplejson import dumps as _simplejson_dumps
from simplejson import loads as simplejson_loads
from ujson import dumps as _ujson_dumps
from ujson import loads as ujson_loads

from orjson import dumps as orjson_dumps
from orjson import loads as orjson_loads


def ujson_dumps(obj):
    return _ujson_dumps(obj).encode("utf-8")


def rapidjson_dumps(obj):
    return _rapidjson_dumps(obj).encode("utf-8")


def json_dumps(obj):
    return _json_dumps(obj).encode("utf-8")


def simplejson_dumps(obj):
    return _simplejson_dumps(obj).encode("utf-8")


libraries = {
    "orjson": (orjson_dumps, orjson_loads),
    "ujson": (ujson_dumps, ujson_loads),
    "json": (json_dumps, json_loads),
    "rapidjson": (rapidjson_dumps, rapidjson_loads),
    "simplejson": (simplejson_dumps, simplejson_loads),
}


fixtures = [
    "canada.json",
    "citm_catalog.json",
    "github.json",
    "twitter.json",
]



# SPDX-License-Identifier: (Apache-2.0 OR MIT)

import dataclasses
import datetime
import gc
import random
from typing import List

try:
    import pytz
except ImportError:
    pytz = None  # type: ignore

try:
    import psutil
except ImportError:
    psutil = None  # type: ignore
import pytest

import orjson

try:
    import numpy
except ImportError:
    numpy = None  # type: ignore

try:
    import pandas
except ImportError:
    pandas = None  # type: ignore

FIXTURE = '{"a":[81891289, 8919812.190129012], "b": false, "c": null, "d": "東京"}'


def default(obj):
    return str(obj)


@dataclasses.dataclass
class Member:
    id: int
    active: bool


@dataclasses.dataclass
class Object:
    id: int
    updated_at: datetime.datetime
    name: str
    members: List[Member]


DATACLASS_FIXTURE = [
    Object(
        i,
        datetime.datetime.now(datetime.timezone.utc)
        + datetime.timedelta(seconds=random.randint(0, 10000)),
        str(i) * 3,
        [Member(j, True) for j in range(0, 10)],
    )
    for i in range(100000, 101000)
]

MAX_INCREASE = 4194304  # 4MiB


class Unsupported:
    pass


class TestMemory:
    @pytest.mark.skipif(psutil is None, reason="psutil not installed")
    def test_memory_loads(self):
        """
        loads() memory leak
        """
        proc = psutil.Process()
        gc.collect()
        val = orjson.loads(FIXTURE)
        assert val
        mem = proc.memory_info().rss
        for _ in range(10000):
            val = orjson.loads(FIXTURE)
            assert val
        gc.collect()
        assert proc.memory_info().rss <= mem + MAX_INCREASE

    @pytest.mark.skipif(psutil is None, reason="psutil not installed")
    def test_memory_loads_memoryview(self):
        """
        loads() memory leak using memoryview
        """
        proc = psutil.Process()
        gc.collect()
        fixture = FIXTURE.encode("utf-8")
        val = orjson.loads(fixture)
        assert val
        mem = proc.memory_info().rss
        for _ in range(10000):
            val = orjson.loads(memoryview(fixture))
            assert val
        gc.collect()
        assert proc.memory_info().rss <= mem + MAX_INCREASE

    @pytest.mark.skipif(psutil is None, reason="psutil not installed")
    def test_memory_dumps(self):
        """
        dumps() memory leak
        """
        proc = psutil.Process()
        gc.collect()
        fixture = orjson.loads(FIXTURE)
        val = orjson.dumps(fixture)
        assert val
        mem = proc.memory_info().rss
        for _ in range(10000):
            val = orjson.dumps(fixture)
            assert val
        gc.collect()
        assert proc.memory_info().rss <= mem + MAX_INCREASE
        assert proc.memory_info().rss <= mem + MAX_INCREASE

    @pytest.mark.skipif(psutil is None, reason="psutil not installed")
    def test_memory_loads_exc(self):
        """
        loads() memory leak exception without a GC pause
        """
        proc = psutil.Process()
        gc.disable()
        mem = proc.memory_info().rss
        n = 10000
        i = 0
        for _ in range(n):
            try:
                orjson.loads("")
            except orjson.JSONDecodeError:
                i += 1
        assert n == i
        assert proc.memory_info().rss <= mem + MAX_INCREASE
        gc.enable()

    @pytest.mark.skipif(psutil is None, reason="psutil not installed")
    def test_memory_dumps_exc(self):
        """
        dumps() memory leak exception without a GC pause
        """
        proc = psutil.Process()
        gc.disable()
        data = Unsupported()
        mem = proc.memory_info().rss
        n = 10000
        i = 0
        for _ in range(n):
            try:
                orjson.dumps(data)
            except orjson.JSONEncodeError:
                i += 1
        assert n == i
        assert proc.memory_info().rss <= mem + MAX_INCREASE
        gc.enable()

    @pytest.mark.skipif(psutil is None, reason="psutil not installed")
    def test_memory_dumps_default(self):
        """
        dumps() default memory leak
        """
        proc = psutil.Process()
        gc.collect()
        fixture = orjson.loads(FIXTURE)

        class Custom:
            def __init__(self, name):
                self.name = name

            def __str__(self):
                return f"{self.__class__.__name__}({self.name})"

        fixture["custom"] = Custom("orjson")
        val = orjson.dumps(fixture, default=default)
        mem = proc.memory_info().rss
        for _ in range(10000):
            val = orjson.dumps(fixture, default=default)
            assert val
        gc.collect()
        assert proc.memory_info().rss <= mem + MAX_INCREASE

    @pytest.mark.skipif(psutil is None, reason="psutil not installed")
    def test_memory_dumps_dataclass(self):
        """
        dumps() dataclass memory leak
        """
        proc = psutil.Process()
        gc.collect()
        val = orjson.dumps(DATACLASS_FIXTURE)
        assert val
        mem = proc.memory_info().rss
        for _ in range(100):
            val = orjson.dumps(DATACLASS_FIXTURE)
            assert val
        assert val
        gc.collect()
        assert proc.memory_info().rss <= mem + MAX_INCREASE

    @pytest.mark.skipif(
        psutil is None or pytz is None,
        reason="psutil not installed",
    )
    def test_memory_dumps_pytz_tzinfo(self):
        """
        dumps() pytz tzinfo memory leak
        """
        proc = psutil.Process()
        gc.collect()
        dt = datetime.datetime.now()
        val = orjson.dumps(pytz.UTC.localize(dt))
        assert val
        mem = proc.memory_info().rss
        for _ in range(50000):
            val = orjson.dumps(pytz.UTC.localize(dt))
            assert val
        assert val
        gc.collect()
        assert proc.memory_info().rss <= mem + MAX_INCREASE

    @pytest.mark.skipif(psutil is None, reason="psutil not installed")
    def test_memory_loads_keys(self):
        """
        loads() memory leak with number of keys causing cache eviction
        """
        proc = psutil.Process()
        gc.collect()
        fixture = {"key_%s" % idx: "value" for idx in range(1024)}
        assert len(fixture) == 1024
        val = orjson.dumps(fixture)
        loaded = orjson.loads(val)
        assert loaded
        mem = proc.memory_info().rss
        for _ in range(100):
            loaded = orjson.loads(val)
            assert loaded
        gc.collect()
        assert proc.memory_info().rss <= mem + MAX_INCREASE

    @pytest.mark.skipif(psutil is None, reason="psutil not installed")
    @pytest.mark.skipif(numpy is None, reason="numpy is not installed")
    def test_memory_dumps_numpy(self):
        """
        dumps() numpy memory leak
        """
        proc = psutil.Process()
        gc.collect()
        fixture = numpy.random.rand(4, 4, 4)
        val = orjson.dumps(fixture, option=orjson.OPT_SERIALIZE_NUMPY)
        assert val
        mem = proc.memory_info().rss
        for _ in range(100):
            val = orjson.dumps(fixture, option=orjson.OPT_SERIALIZE_NUMPY)
            assert val
        assert val
        gc.collect()
        assert proc.memory_info().rss <= mem + MAX_INCREASE

    @pytest.mark.skipif(psutil is None, reason="psutil not installed")
    @pytest.mark.skipif(pandas is None, reason="pandas is not installed")
    def test_memory_dumps_pandas(self):
        """
        dumps() pandas memory leak
        """
        proc = psutil.Process()
        gc.collect()
        numpy.random.rand(4, 4, 4)
        df = pandas.Series(numpy.random.rand(4, 4, 4).tolist())
        val = df.map(orjson.dumps)
        assert not val.empty
        mem = proc.memory_info().rss
        for _ in range(100):
            val = df.map(orjson.dumps)
            assert not val.empty
        gc.collect()
        assert proc.memory_info().rss <= mem + MAX_INCREASE

    @pytest.mark.skipif(psutil is None, reason="psutil not installed")
    def test_memory_dumps_fragment(self):
        """
        dumps() Fragment memory leak
        """
        proc = psutil.Process()
        gc.collect()
        orjson.dumps(orjson.Fragment(str(0)))
        mem = proc.memory_info().rss
        for i in range(10000):
            orjson.dumps(orjson.Fragment(str(i)))
        gc.collect()
        assert proc.memory_info().rss <= mem + MAX_INCREASE



# SPDX-License-Identifier: (Apache-2.0 OR MIT)

import orjson

from .util import read_fixture_obj


class TestDictSortKeys:
    # citm_catalog is already sorted
    def test_twitter_sorted(self):
        """
        twitter.json sorted
        """
        obj = read_fixture_obj("twitter.json.xz")
        assert list(obj.keys()) != sorted(list(obj.keys()))
        serialized = orjson.dumps(obj, option=orjson.OPT_SORT_KEYS)
        val = orjson.loads(serialized)
        assert list(val.keys()) == sorted(list(val.keys()))

    def test_canada_sorted(self):
        """
        canada.json sorted
        """
        obj = read_fixture_obj("canada.json.xz")
        assert list(obj.keys()) != sorted(list(obj.keys()))
        serialized = orjson.dumps(obj, option=orjson.OPT_SORT_KEYS)
        val = orjson.loads(serialized)
        assert list(val.keys()) == sorted(list(val.keys()))

    def test_github_sorted(self):
        """
        github.json sorted
        """
        obj = read_fixture_obj("github.json.xz")
        for each in obj:
            assert list(each.keys()) != sorted(list(each.keys()))
        serialized = orjson.dumps(obj, option=orjson.OPT_SORT_KEYS)
        val = orjson.loads(serialized)
        for each in val:
            assert list(each.keys()) == sorted(list(each.keys()))

    def test_utf8_sorted(self):
        """
        UTF-8 sorted
        """
        obj = {"a": 1, "ä": 2, "A": 3}
        assert list(obj.keys()) != sorted(list(obj.keys()))
        serialized = orjson.dumps(obj, option=orjson.OPT_SORT_KEYS)
        val = orjson.loads(serialized)
        assert list(val.keys()) == sorted(list(val.keys()))



# SPDX-License-Identifier: (Apache-2.0 OR MIT)

from json import loads as json_loads

import pytest

from .data import libraries


@pytest.mark.parametrize("data", ["[]", "{}", '""'])
@pytest.mark.parametrize("library", libraries)
def test_empty(benchmark, data, library):
    dumper, loader = libraries[library]
    correct = json_loads(dumper(loader(data))) == json_loads(data)  # type: ignore
    benchmark.extra_info["correct"] = correct
    benchmark(loader, data)



# SPDX-License-Identifier: (Apache-2.0 OR MIT)

import random

import pytest

import orjson

try:
    from faker import Faker
except ImportError:
    Faker = None  # type: ignore

NUM_LOOPS = 10
NUM_SHUFFLES = 10
NUM_ENTRIES = 250

FAKER_LOCALES = [
    "ar_AA",
    "fi_FI",
    "fil_PH",
    "he_IL",
    "ja_JP",
    "th_TH",
    "tr_TR",
    "uk_UA",
    "vi_VN",
]


class TestFaker:
    @pytest.mark.skipif(Faker is None, reason="faker not available")
    def test_faker(self):
        fake = Faker(FAKER_LOCALES)
        profile_keys = list(
            set(fake.profile().keys()) - {"birthdate", "current_location"}
        )
        for _ in range(0, NUM_LOOPS):
            data = [
                {
                    "person": fake.profile(profile_keys),
                    "emoji": fake.emoji(),
                    "text": fake.paragraphs(),
                }
                for _ in range(0, NUM_ENTRIES)
            ]
            for _ in range(0, NUM_SHUFFLES):
                random.shuffle(data)
                output = orjson.dumps(data)
                assert orjson.loads(output) == data



# SPDX-License-Identifier: (Apache-2.0 OR MIT)

import orjson

from .util import read_fixture_obj


class TestAppendNewline:
    def test_dumps_newline(self):
        """
        dumps() OPT_APPEND_NEWLINE
        """
        assert orjson.dumps([], option=orjson.OPT_APPEND_NEWLINE) == b"[]\n"

    def test_twitter_newline(self):
        """
        loads(),dumps() twitter.json OPT_APPEND_NEWLINE
        """
        val = read_fixture_obj("twitter.json.xz")
        assert orjson.loads(orjson.dumps(val, option=orjson.OPT_APPEND_NEWLINE)) == val

    def test_canada(self):
        """
        loads(), dumps() canada.json OPT_APPEND_NEWLINE
        """
        val = read_fixture_obj("canada.json.xz")
        assert orjson.loads(orjson.dumps(val, option=orjson.OPT_APPEND_NEWLINE)) == val

    def test_citm_catalog_newline(self):
        """
        loads(), dumps() citm_catalog.json OPT_APPEND_NEWLINE
        """
        val = read_fixture_obj("citm_catalog.json.xz")
        assert orjson.loads(orjson.dumps(val, option=orjson.OPT_APPEND_NEWLINE)) == val

    def test_github_newline(self):
        """
        loads(), dumps() github.json OPT_APPEND_NEWLINE
        """
        val = read_fixture_obj("github.json.xz")
        assert orjson.loads(orjson.dumps(val, option=orjson.OPT_APPEND_NEWLINE)) == val



# SPDX-License-Identifier: (Apache-2.0 OR MIT)

from datetime import datetime, timezone
from uuid import uuid4

from flask import Flask

import orjson

app = Flask(__name__)

NOW = datetime.now(timezone.utc)


@app.route("/")
def root():
    data = {
        "uuid": uuid4(),
        "updated_at": NOW,
        "data": [1, 2.2, None, True, False, orjson.Fragment(b"{}")],
    }
    payload = orjson.dumps(
        data, option=orjson.OPT_NAIVE_UTC | orjson.OPT_OMIT_MICROSECONDS
    )
    return app.response_class(
        response=payload,
        status=200,
        mimetype="application/json; charset=utf-8",
    )



import orjson


class C:
    c: "C"

    def __del__(self):
        orjson.loads('"' + "a" * 10000 + '"')


def test_reentrant():
    c = C()
    c.c = c
    del c

    orjson.loads("[" + "[]," * 1000 + "[]]")



# SPDX-License-Identifier: (Apache-2.0 OR MIT)

from json import loads as json_loads

import pytest

from .data import fixtures, libraries
from .util import read_fixture


@pytest.mark.parametrize("fixture", fixtures)
@pytest.mark.parametrize("library", libraries)
def test_loads(benchmark, fixture, library):
    dumper, loader = libraries[library]
    benchmark.group = f"{fixture} deserialization"
    benchmark.extra_info["lib"] = library
    data = read_fixture(f"{fixture}.xz")
    correct = json_loads(dumper(loader(data))) == json_loads(data)  # type: ignore
    benchmark.extra_info["correct"] = correct
    benchmark(loader, data)



# SPDX-License-Identifier: (Apache-2.0 OR MIT)

import uuid

import pytest

import orjson


class TestUUID:
    def test_uuid_immutable(self):
        """
        UUID objects are immutable
        """
        val = uuid.uuid4()
        with pytest.raises(TypeError):
            val.int = 1  # type: ignore
        with pytest.raises(TypeError):
            val.int = None  # type: ignore

    def test_uuid_int(self):
        """
        UUID.int is a 128-bit integer
        """
        val = uuid.UUID("7202d115-7ff3-4c81-a7c1-2a1f067b1ece")
        assert isinstance(val.int, int)
        assert val.int >= 2**64
        assert val.int < 2**128
        assert val.int, 151546616840194781678008611711208857294

    def test_uuid_overflow(self):
        """
        UUID.int can't trigger errors in _PyLong_AsByteArray
        """
        with pytest.raises(ValueError):
            uuid.UUID(int=2**128)
        with pytest.raises(ValueError):
            uuid.UUID(int=-1)

    def test_uuid_subclass(self):
        """
        UUID subclasses are not serialized
        """

        class AUUID(uuid.UUID):
            pass

        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(AUUID("{12345678-1234-5678-1234-567812345678}"))

    def test_serializes_withopt(self):
        """
        dumps() accepts deprecated OPT_SERIALIZE_UUID
        """
        assert (
            orjson.dumps(
                uuid.UUID("7202d115-7ff3-4c81-a7c1-2a1f067b1ece"),
                option=orjson.OPT_SERIALIZE_UUID,
            )
            == b'"7202d115-7ff3-4c81-a7c1-2a1f067b1ece"'
        )

    def test_nil_uuid(self):
        assert (
            orjson.dumps(uuid.UUID("00000000-0000-0000-0000-000000000000"))
            == b'"00000000-0000-0000-0000-000000000000"'
        )

    def test_all_ways_to_create_uuid_behave_equivalently(self):
        # Note that according to the docstring for the uuid.UUID class, all the
        # forms below are equivalent -- they end up with the same value for
        # `self.int`, which is all that really matters
        uuids = [
            uuid.UUID("{12345678-1234-5678-1234-567812345678}"),
            uuid.UUID("12345678123456781234567812345678"),
            uuid.UUID("urn:uuid:12345678-1234-5678-1234-567812345678"),
            uuid.UUID(bytes=b"\x12\x34\x56\x78" * 4),
            uuid.UUID(
                bytes_le=b"\x78\x56\x34\x12\x34\x12\x78\x56"
                + b"\x12\x34\x56\x78\x12\x34\x56\x78"
            ),
            uuid.UUID(fields=(0x12345678, 0x1234, 0x5678, 0x12, 0x34, 0x567812345678)),
            uuid.UUID(int=0x12345678123456781234567812345678),
        ]
        result = orjson.dumps(uuids)
        canonical_uuids = ['"%s"' % str(u) for u in uuids]
        serialized = ("[%s]" % ",".join(canonical_uuids)).encode("utf8")
        assert result == serialized

    def test_serializes_correctly_with_leading_zeroes(self):
        instance = uuid.UUID(int=0x00345678123456781234567812345678)
        assert orjson.dumps(instance) == ('"%s"' % str(instance)).encode("utf8")

    def test_all_uuid_creation_functions_create_serializable_uuids(self):
        uuids = (
            uuid.uuid1(),
            uuid.uuid3(uuid.NAMESPACE_DNS, "python.org"),
            uuid.uuid4(),
            uuid.uuid5(uuid.NAMESPACE_DNS, "python.org"),
        )
        for val in uuids:
            assert orjson.dumps(val) == f'"{val}"'.encode("utf-8")



# SPDX-License-Identifier: (Apache-2.0 OR MIT)

import json

import pytest

import orjson

from .util import read_fixture_str

ASCII_TEST = b"""\
{
  "a": "qwe",
  "b": "qweqwe",
  "c": "qweq",
  "d: "qwe"
}
"""

MULTILINE_EMOJI = """[
    "😊",
    "a"
"""


class TestJsonDecodeError:
    def _get_error_infos(self, json_decode_error_exc_info):
        return {
            k: v
            for k, v in json_decode_error_exc_info.value.__dict__.items()
            if k in ("pos", "lineno", "colno")
        }

    def _test(self, data, expected_err_infos):
        with pytest.raises(json.decoder.JSONDecodeError) as json_exc_info:
            json.loads(data)

        with pytest.raises(json.decoder.JSONDecodeError) as orjson_exc_info:
            orjson.loads(data)

        assert (
            self._get_error_infos(json_exc_info)
            == self._get_error_infos(orjson_exc_info)
            == expected_err_infos
        )

    def test_empty(self):
        with pytest.raises(orjson.JSONDecodeError) as json_exc_info:
            orjson.loads("")
        assert str(json_exc_info.value).startswith(
            "Input is a zero-length, empty document:"
        )

    def test_ascii(self):
        self._test(
            ASCII_TEST,
            {"pos": 55, "lineno": 5, "colno": 8},
        )

    def test_latin1(self):
        self._test(
            """["üýþÿ", "a" """,
            {"pos": 13, "lineno": 1, "colno": 14},
        )

    def test_two_byte_str(self):
        self._test(
            """["東京", "a" """,
            {"pos": 11, "lineno": 1, "colno": 12},
        )

    def test_two_byte_bytes(self):
        self._test(
            b'["\xe6\x9d\xb1\xe4\xba\xac", "a" ',
            {"pos": 11, "lineno": 1, "colno": 12},
        )

    def test_four_byte(self):
        self._test(
            MULTILINE_EMOJI,
            {"pos": 19, "lineno": 4, "colno": 1},
        )

    def test_tab(self):
        data = read_fixture_str("fail26.json", "jsonchecker")
        with pytest.raises(json.decoder.JSONDecodeError) as json_exc_info:
            json.loads(data)

        assert self._get_error_infos(json_exc_info) == {
            "pos": 5,
            "lineno": 1,
            "colno": 6,
        }

        with pytest.raises(json.decoder.JSONDecodeError) as json_exc_info:
            orjson.loads(data)

        assert self._get_error_infos(json_exc_info) == {
            "pos": 6,
            "lineno": 1,
            "colno": 7,
        }


class Custom:
    pass


class CustomException(Exception):
    pass


def default_typeerror(obj):
    raise TypeError


def default_notimplementederror(obj):
    raise NotImplementedError


def default_systemerror(obj):
    raise SystemError


def default_importerror(obj):
    import doesnotexist

    assert doesnotexist


CUSTOM_ERROR_MESSAGE = "zxc"


def default_customerror(obj):
    raise CustomException(CUSTOM_ERROR_MESSAGE)


class TestJsonEncodeError:
    def test_dumps_arg(self):
        with pytest.raises(orjson.JSONEncodeError) as exc_info:
            orjson.dumps()  # type: ignore
        assert exc_info.type == orjson.JSONEncodeError
        assert (
            str(exc_info.value)
            == "dumps() missing 1 required positional argument: 'obj'"
        )
        assert exc_info.value.__cause__ is None

    def test_dumps_chain_none(self):
        with pytest.raises(orjson.JSONEncodeError) as exc_info:
            orjson.dumps(Custom())
        assert exc_info.type == orjson.JSONEncodeError
        assert str(exc_info.value) == "Type is not JSON serializable: Custom"
        assert exc_info.value.__cause__ is None

    def test_dumps_chain_u64(self):
        with pytest.raises(orjson.JSONEncodeError) as exc_info:
            orjson.dumps([18446744073709551615, Custom()])
        assert exc_info.type == orjson.JSONEncodeError
        assert exc_info.value.__cause__ is None

    def test_dumps_chain_default_typeerror(self):
        with pytest.raises(orjson.JSONEncodeError) as exc_info:
            orjson.dumps(Custom(), default=default_typeerror)
        assert exc_info.type == orjson.JSONEncodeError
        assert isinstance(exc_info.value.__cause__, TypeError)

    def test_dumps_chain_default_systemerror(self):
        with pytest.raises(orjson.JSONEncodeError) as exc_info:
            orjson.dumps(Custom(), default=default_systemerror)
        assert exc_info.type == orjson.JSONEncodeError
        assert isinstance(exc_info.value.__cause__, SystemError)

    def test_dumps_chain_default_importerror(self):
        with pytest.raises(orjson.JSONEncodeError) as exc_info:
            orjson.dumps(Custom(), default=default_importerror)
        assert exc_info.type == orjson.JSONEncodeError
        assert isinstance(exc_info.value.__cause__, ImportError)

    def test_dumps_chain_default_customerror(self):
        with pytest.raises(orjson.JSONEncodeError) as exc_info:
            orjson.dumps(Custom(), default=default_customerror)
        assert exc_info.type == orjson.JSONEncodeError
        assert isinstance(exc_info.value.__cause__, CustomException)
        assert str(exc_info.value.__cause__) == CUSTOM_ERROR_MESSAGE

    def test_dumps_normalize_exception(self):
        with pytest.raises(orjson.JSONEncodeError) as exc_info:
            orjson.dumps(10**60)
        assert exc_info.type == orjson.JSONEncodeError
        assert isinstance(exc_info.value.__cause__, OverflowError)



# SPDX-License-Identifier: (Apache-2.0 OR MIT)

import datetime
import sys
import uuid

import pytest

import orjson

try:
    import numpy
except ImportError:
    numpy = None  # type: ignore


class Custom:
    def __init__(self):
        self.name = uuid.uuid4().hex

    def __str__(self):
        return f"{self.__class__.__name__}({self.name})"


class Recursive:
    def __init__(self, cur):
        self.cur = cur


def default_recursive(obj):
    if obj.cur != 0:
        obj.cur -= 1
        return obj
    return obj.cur


def default_raises(obj):
    raise TypeError


class TestType:
    def test_default_not_callable(self):
        """
        dumps() default not callable
        """
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(Custom(), default=NotImplementedError)

        ran = False
        try:
            orjson.dumps(Custom(), default=NotImplementedError)
        except Exception as err:
            assert isinstance(err, orjson.JSONEncodeError)
            assert str(err) == "default serializer exceeds recursion limit"
            ran = True
        assert ran

    def test_default_func(self):
        """
        dumps() default function
        """
        ref = Custom()

        def default(obj):
            return str(obj)

        assert orjson.dumps(ref, default=default) == b'"%s"' % str(ref).encode("utf-8")

    def test_default_func_none(self):
        """
        dumps() default function None ok
        """
        assert orjson.dumps(Custom(), default=lambda x: None) == b"null"

    def test_default_func_empty(self):
        """
        dumps() default function no explicit return
        """
        ref = Custom()

        def default(obj):
            if isinstance(obj, set):
                return list(obj)

        assert orjson.dumps(ref, default=default) == b"null"
        assert orjson.dumps({ref}, default=default) == b"[null]"

    def test_default_func_exc(self):
        """
        dumps() default function raises exception
        """

        def default(obj):
            raise NotImplementedError

        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(Custom(), default=default)

        ran = False
        try:
            orjson.dumps(Custom(), default=default)
        except Exception as err:
            assert isinstance(err, orjson.JSONEncodeError)
            assert str(err) == "Type is not JSON serializable: Custom"
            ran = True
        assert ran

    def test_default_exception_type(self):
        """
        dumps() TypeError in default() raises orjson.JSONEncodeError
        """
        ref = Custom()

        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(ref, default=default_raises)

    def test_default_vectorcall_str(self):
        """
        dumps() default function vectorcall str
        """

        class SubStr(str):
            pass

        obj = SubStr("saasa")
        ref = b'"%s"' % str(obj).encode("utf-8")
        assert (
            orjson.dumps(obj, option=orjson.OPT_PASSTHROUGH_SUBCLASS, default=str)
            == ref
        )

    def test_default_vectorcall_list(self):
        """
        dumps() default function vectorcall list
        """
        obj = {1, 2}
        ref = b"[1,2]"
        assert orjson.dumps(obj, default=list) == ref

    def test_default_func_nested_str(self):
        """
        dumps() default function nested str
        """
        ref = Custom()

        def default(obj):
            return str(obj)

        assert orjson.dumps({"a": ref}, default=default) == b'{"a":"%s"}' % str(
            ref
        ).encode("utf-8")

    def test_default_func_list(self):
        """
        dumps() default function nested list
        """
        ref = Custom()

        def default(obj):
            if isinstance(obj, Custom):
                return [str(obj)]

        assert orjson.dumps({"a": ref}, default=default) == b'{"a":["%s"]}' % str(
            ref
        ).encode("utf-8")

    def test_default_func_nested_list(self):
        """
        dumps() default function list
        """
        ref = Custom()

        def default(obj):
            return str(obj)

        assert orjson.dumps([ref] * 100, default=default) == b"[%s]" % b",".join(
            b'"%s"' % str(ref).encode("utf-8") for _ in range(100)
        )

    def test_default_func_bytes(self):
        """
        dumps() default function errors on non-str
        """
        ref = Custom()

        def default(obj):
            return bytes(obj)

        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(ref, default=default)

        ran = False
        try:
            orjson.dumps(ref, default=default)
        except Exception as err:
            assert isinstance(err, orjson.JSONEncodeError)
            assert str(err) == "Type is not JSON serializable: Custom"
            ran = True
        assert ran

    def test_default_func_invalid_str(self):
        """
        dumps() default function errors on invalid str
        """
        ref = Custom()

        def default(obj):
            return "\ud800"

        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(ref, default=default)

    def test_default_lambda_ok(self):
        """
        dumps() default lambda
        """
        ref = Custom()
        assert orjson.dumps(ref, default=lambda x: str(x)) == b'"%s"' % str(ref).encode(
            "utf-8"
        )

    def test_default_callable_ok(self):
        """
        dumps() default callable
        """

        class CustomSerializer:
            def __init__(self):
                self._cache = {}

            def __call__(self, obj):
                if obj not in self._cache:
                    self._cache[obj] = str(obj)
                return self._cache[obj]

        ref_obj = Custom()
        ref_bytes = b'"%s"' % str(ref_obj).encode("utf-8")
        for obj in [ref_obj] * 100:
            assert orjson.dumps(obj, default=CustomSerializer()) == ref_bytes

    def test_default_recursion(self):
        """
        dumps() default recursion limit
        """
        assert orjson.dumps(Recursive(254), default=default_recursive) == b"0"

    def test_default_recursion_reset(self):
        """
        dumps() default recursion limit reset
        """
        assert (
            orjson.dumps(
                [Recursive(254), {"a": "b"}, Recursive(254), Recursive(254)],
                default=default_recursive,
            )
            == b'[0,{"a":"b"},0,0]'
        )

    def test_default_recursion_infinite(self):
        """
        dumps() default infinite recursion
        """
        ref = Custom()

        def default(obj):
            return obj

        refcount = sys.getrefcount(ref)
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(ref, default=default)
        assert sys.getrefcount(ref) == refcount

    def test_reference_cleanup_default_custom_pass(self):
        ref = Custom()

        def default(obj):
            if isinstance(ref, Custom):
                return str(ref)
            raise TypeError

        refcount = sys.getrefcount(ref)
        orjson.dumps(ref, default=default)
        assert sys.getrefcount(ref) == refcount

    def test_reference_cleanup_default_custom_error(self):
        """
        references to encoded objects are cleaned up
        """
        ref = Custom()

        def default(obj):
            raise TypeError

        refcount = sys.getrefcount(ref)
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(ref, default=default)
        assert sys.getrefcount(ref) == refcount

    def test_reference_cleanup_default_subclass(self):
        ref = datetime.datetime(1970, 1, 1, 0, 0, 0)

        def default(obj):
            if isinstance(ref, datetime.datetime):
                return repr(ref)
            raise TypeError

        refcount = sys.getrefcount(ref)
        orjson.dumps(ref, option=orjson.OPT_PASSTHROUGH_DATETIME, default=default)
        assert sys.getrefcount(ref) == refcount

    def test_reference_cleanup_default_subclass_lambda(self):
        ref = uuid.uuid4()

        refcount = sys.getrefcount(ref)
        orjson.dumps(
            ref, option=orjson.OPT_PASSTHROUGH_DATETIME, default=lambda val: str(val)
        )
        assert sys.getrefcount(ref) == refcount

    @pytest.mark.skipif(numpy is None, reason="numpy is not installed")
    def test_default_numpy(self):
        ref = numpy.array([""] * 100)
        refcount = sys.getrefcount(ref)
        orjson.dumps(
            ref, option=orjson.OPT_SERIALIZE_NUMPY, default=lambda val: val.tolist()
        )
        assert sys.getrefcount(ref) == refcount

    def test_default_set(self):
        """
        dumps() default function with set
        """

        def default(obj):
            if isinstance(obj, set):
                return list(obj)
            raise TypeError

        assert orjson.dumps({1, 2}, default=default) == b"[1,2]"



# SPDX-License-Identifier: (Apache-2.0 OR MIT)

import orjson


class TestCanonicalTests:
    def test_dumps_ctrl_escape(self):
        """
        dumps() ctrl characters
        """
        assert orjson.dumps("text\u0003\r\n") == b'"text\\u0003\\r\\n"'

    def test_dumps_escape_quote_backslash(self):
        """
        dumps() quote, backslash escape
        """
        assert orjson.dumps(r'"\ test') == b'"\\"\\\\ test"'

    def test_dumps_escape_line_separator(self):
        """
        dumps() U+2028, U+2029 escape
        """
        assert (
            orjson.dumps({"spaces": "\u2028 \u2029"})
            == b'{"spaces":"\xe2\x80\xa8 \xe2\x80\xa9"}'
        )



# SPDX-License-Identifier: (Apache-2.0 OR MIT)

import lzma
import os
from functools import lru_cache
from pathlib import Path
from typing import Any

import orjson

dirname = os.path.join(os.path.dirname(__file__), "../data")

if hasattr(os, "sched_setaffinity"):
    os.sched_setaffinity(os.getpid(), {0, 1})


@lru_cache(maxsize=None)
def read_fixture(filename: str) -> bytes:
    path = Path(dirname, filename)
    if path.suffix == ".xz":
        contents = lzma.decompress(path.read_bytes())
    else:
        contents = path.read_bytes()
    return contents


@lru_cache(maxsize=None)
def read_fixture_obj(filename: str) -> Any:
    return orjson.loads(read_fixture(filename))



# SPDX-License-Identifier: (Apache-2.0 OR MIT)

import pytest

import orjson

from .util import read_fixture_bytes


def _read_file(filename):
    return read_fixture_bytes(filename, "transform").strip(b"\n").strip(b"\r")


class TestJSONTestSuiteTransform:
    def _pass_transform(self, filename, reference=None):
        data = _read_file(filename)
        assert orjson.dumps(orjson.loads(data)) == (reference or data)

    def _fail_transform(self, filename):
        data = _read_file(filename)
        with pytest.raises(orjson.JSONDecodeError):
            orjson.loads(data)

    def test_number_1(self):
        """
        number_1.0.json
        """
        self._pass_transform("number_1.0.json")

    def test_number_1e6(self):
        """
        number_1e6.json
        """
        self._pass_transform("number_1e6.json", b"[1000000.0]")

    def test_number_1e_999(self):
        """
        number_1e-999.json
        """
        self._pass_transform("number_1e-999.json", b"[0.0]")

    def test_number_10000000000000000999(self):
        """
        number_10000000000000000999.json
        """
        # cannot serialize due to range
        assert orjson.loads(_read_file("number_10000000000000000999.json")) == [
            10000000000000000999
        ]

    def test_number_1000000000000000(self):
        """
        number_1000000000000000.json
        """
        self._pass_transform("number_1000000000000000.json")

    def test_object_key_nfc_nfd(self):
        """
        object_key_nfc_nfd.json
        """
        self._pass_transform("object_key_nfc_nfd.json")

    def test_object_key_nfd_nfc(self):
        """
        object_key_nfd_nfc.json
        """
        self._pass_transform("object_key_nfd_nfc.json")

    def test_object_same_key_different_values(self):
        """
        object_same_key_different_values.json
        """
        self._pass_transform("object_same_key_different_values.json", b'{"a":2}')

    def test_object_same_key_same_value(self):
        """
        object_same_key_same_value.json
        """
        self._pass_transform("object_same_key_same_value.json", b'{"a":1}')

    def test_object_same_key_unclear_values(self):
        """
        object_same_key_unclear_values.json
        """
        data = _read_file("object_same_key_unclear_values.json")
        # varies by backend
        assert data in (b'{"a":-0.0}', b'{"a":0, "a":-0}')

    def test_string_1_escaped_invalid_codepoint(self):
        """
        string_1_escaped_invalid_codepoint.json
        """
        self._fail_transform("string_1_escaped_invalid_codepoint.json")

    def test_string_1_invalid_codepoint(self):
        """
        string_1_invalid_codepoint.json
        """
        self._fail_transform("string_1_invalid_codepoint.json")

    def test_string_2_escaped_invalid_codepoints(self):
        """
        string_2_escaped_invalid_codepoints.json
        """
        self._fail_transform("string_2_escaped_invalid_codepoints.json")

    def test_string_2_invalid_codepoints(self):
        """
        string_2_invalid_codepoints.json
        """
        self._fail_transform("string_2_invalid_codepoints.json")

    def test_string_3_escaped_invalid_codepoints(self):
        """
        string_3_escaped_invalid_codepoints.json
        """
        self._fail_transform("string_3_escaped_invalid_codepoints.json")

    def test_string_3_invalid_codepoints(self):
        """
        string_3_invalid_codepoints.json
        """
        self._fail_transform("string_3_invalid_codepoints.json")

    def test_string_with_escaped_NULL(self):
        """
        string_with_escaped_NULL.json
        """
        self._pass_transform("string_with_escaped_NULL.json")



# SPDX-License-Identifier: (Apache-2.0 OR MIT)

import pytest

import orjson

try:
    import pandas
except ImportError:
    pandas = None  # type: ignore

from .util import read_fixture_bytes


class TestFragment:
    def test_fragment_fragment_eq(self):
        assert orjson.Fragment(b"{}") != orjson.Fragment(b"{}")

    def test_fragment_fragment_not_mut(self):
        fragment = orjson.Fragment(b"{}")
        with pytest.raises(AttributeError):
            fragment.contents = b"[]"
        assert orjson.dumps(fragment) == b"{}"

    def test_fragment_repr(self):
        assert repr(orjson.Fragment(b"{}")).startswith("<orjson.Fragment object at ")

    def test_fragment_fragment_bytes(self):
        assert orjson.dumps(orjson.Fragment(b"{}")) == b"{}"
        assert orjson.dumps(orjson.Fragment(b"[]")) == b"[]"
        assert orjson.dumps([orjson.Fragment(b"{}")]) == b"[{}]"
        assert orjson.dumps([orjson.Fragment(b'{}"a\\')]) == b'[{}"a\\]'

    def test_fragment_fragment_str(self):
        assert orjson.dumps(orjson.Fragment("{}")) == b"{}"
        assert orjson.dumps(orjson.Fragment("[]")) == b"[]"
        assert orjson.dumps([orjson.Fragment("{}")]) == b"[{}]"
        assert orjson.dumps([orjson.Fragment('{}"a\\')]) == b'[{}"a\\]'

    def test_fragment_fragment_str_empty(self):
        assert orjson.dumps(orjson.Fragment("")) == b""

    def test_fragment_fragment_str_str(self):
        assert orjson.dumps(orjson.Fragment('"str"')) == b'"str"'

    def test_fragment_fragment_str_emoji(self):
        assert orjson.dumps(orjson.Fragment('"🐈"')) == b'"\xf0\x9f\x90\x88"'

    def test_fragment_fragment_str_array(self):
        n = 8096
        obj = [orjson.Fragment('"🐈"')] * n
        ref = b"[" + b",".join((b'"\xf0\x9f\x90\x88"' for _ in range(0, n))) + b"]"
        assert orjson.dumps(obj) == ref

    def test_fragment_fragment_str_invalid(self):
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps(orjson.Fragment("\ud800"))  # type: ignore

    def test_fragment_fragment_bytes_invalid(self):
        assert orjson.dumps(orjson.Fragment(b"\\ud800")) == b"\\ud800"

    def test_fragment_fragment_none(self):
        with pytest.raises(orjson.JSONEncodeError):
            orjson.dumps([orjson.Fragment(None)])  # type: ignore

    def test_fragment_fragment_args_zero(self):
        with pytest.raises(TypeError):
            orjson.dumps(orjson.Fragment())

    def test_fragment_fragment_args_two(self):
        with pytest.raises(TypeError):
            orjson.dumps(orjson.Fragment(b"{}", None))  # type: ignore

    def test_fragment_fragment_keywords(self):
        with pytest.raises(TypeError):
            orjson.dumps(orjson.Fragment(contents=b"{}"))  # type: ignore

    def test_fragment_fragment_arg_and_keywords(self):
        with pytest.raises(TypeError):
            orjson.dumps(orjson.Fragment(b"{}", contents=b"{}"))  # type: ignore


@pytest.mark.skipif(pandas is None, reason="pandas is not installed")
class TestFragmentPandas:
    def test_fragment_pandas(self):
        """
        Fragment pandas.DataFrame.to_json()
        """

        def default(value):
            if isinstance(value, pandas.DataFrame):
                return orjson.Fragment(value.to_json(orient="records"))
            raise TypeError

        val = pandas.DataFrame({"foo": [1, 2, 3], "bar": [4, 5, 6]})
        assert (
            orjson.dumps({"data": val}, default=default)
            == b'{"data":[{"foo":1,"bar":4},{"foo":2,"bar":5},{"foo":3,"bar":6}]}'
        )


class TestFragmentParsing:
    def _run_test(self, filename: str):
        data = read_fixture_bytes(filename, "parsing")
        orjson.dumps(orjson.Fragment(data))

    def test_fragment_y_array_arraysWithSpace(self):
        self._run_test("y_array_arraysWithSpaces.json")

    def test_fragment_y_array_empty_string(self):
        self._run_test("y_array_empty-string.json")

    def test_fragment_y_array_empty(self):
        self._run_test("y_array_empty.json")

    def test_fragment_y_array_ending_with_newline(self):
        self._run_test("y_array_ending_with_newline.json")

    def test_fragment_y_array_false(self):
        self._run_test("y_array_false.json")

    def test_fragment_y_array_heterogeneou(self):
        self._run_test("y_array_heterogeneous.json")

    def test_fragment_y_array_null(self):
        self._run_test("y_array_null.json")

    def test_fragment_y_array_with_1_and_newline(self):
        self._run_test("y_array_with_1_and_newline.json")

    def test_fragment_y_array_with_leading_space(self):
        self._run_test("y_array_with_leading_space.json")

    def test_fragment_y_array_with_several_null(self):
        self._run_test("y_array_with_several_null.json")

    def test_fragment_y_array_with_trailing_space(self):
        self._run_test("y_array_with_trailing_space.json")

    def test_fragment_y_number(self):
        self._run_test("y_number.json")

    def test_fragment_y_number_0e_1(self):
        self._run_test("y_number_0e+1.json")

    def test_fragment_y_number_0e1(self):
        self._run_test("y_number_0e1.json")

    def test_fragment_y_number_after_space(self):
        self._run_test("y_number_after_space.json")

    def test_fragment_y_number_double_close_to_zer(self):
        self._run_test("y_number_double_close_to_zero.json")

    def test_fragment_y_number_int_with_exp(self):
        self._run_test("y_number_int_with_exp.json")

    def test_fragment_y_number_minus_zer(self):
        self._run_test("y_number_minus_zero.json")

    def test_fragment_y_number_negative_int(self):
        self._run_test("y_number_negative_int.json")

    def test_fragment_y_number_negative_one(self):
        self._run_test("y_number_negative_one.json")

    def test_fragment_y_number_negative_zer(self):
        self._run_test("y_number_negative_zero.json")

    def test_fragment_y_number_real_capital_e(self):
        self._run_test("y_number_real_capital_e.json")

    def test_fragment_y_number_real_capital_e_neg_exp(self):
        self._run_test("y_number_real_capital_e_neg_exp.json")

    def test_fragment_y_number_real_capital_e_pos_exp(self):
        self._run_test("y_number_real_capital_e_pos_exp.json")

    def test_fragment_y_number_real_exponent(self):
        self._run_test("y_number_real_exponent.json")

    def test_fragment_y_number_real_fraction_exponent(self):
        self._run_test("y_number_real_fraction_exponent.json")

    def test_fragment_y_number_real_neg_exp(self):
        self._run_test("y_number_real_neg_exp.json")

    def test_fragment_y_number_real_pos_exponent(self):
        self._run_test("y_number_real_pos_exponent.json")

    def test_fragment_y_number_simple_int(self):
        self._run_test("y_number_simple_int.json")

    def test_fragment_y_number_simple_real(self):
        self._run_test("y_number_simple_real.json")

    def test_fragment_y_object(self):
        self._run_test("y_object.json")

    def test_fragment_y_object_basic(self):
        self._run_test("y_object_basic.json")

    def test_fragment_y_object_duplicated_key(self):
        self._run_test("y_object_duplicated_key.json")

    def test_fragment_y_object_duplicated_key_and_value(self):
        self._run_test("y_object_duplicated_key_and_value.json")

    def test_fragment_y_object_empty(self):
        self._run_test("y_object_empty.json")

    def test_fragment_y_object_empty_key(self):
        self._run_test("y_object_empty_key.json")

    def test_fragment_y_object_escaped_null_in_key(self):
        self._run_test("y_object_escaped_null_in_key.json")

    def test_fragment_y_object_extreme_number(self):
        self._run_test("y_object_extreme_numbers.json")

    def test_fragment_y_object_long_string(self):
        self._run_test("y_object_long_strings.json")

    def test_fragment_y_object_simple(self):
        self._run_test("y_object_simple.json")

    def test_fragment_y_object_string_unicode(self):
        self._run_test("y_object_string_unicode.json")

    def test_fragment_y_object_with_newline(self):
        self._run_test("y_object_with_newlines.json")

    def test_fragment_y_string_1_2_3_bytes_UTF_8_sequence(self):
        self._run_test("y_string_1_2_3_bytes_UTF-8_sequences.json")

    def test_fragment_y_string_accepted_surrogate_pair(self):
        self._run_test("y_string_accepted_surrogate_pair.json")

    def test_fragment_y_string_accepted_surrogate_pairs(self):
        self._run_test("y_string_accepted_surrogate_pairs.json")

    def test_fragment_y_string_allowed_escape(self):
        self._run_test("y_string_allowed_escapes.json")

    def test_fragment_y_string_backslash_and_u_escaped_zer(self):
        self._run_test("y_string_backslash_and_u_escaped_zero.json")

    def test_fragment_y_string_backslash_doublequote(self):
        self._run_test("y_string_backslash_doublequotes.json")

    def test_fragment_y_string_comment(self):
        self._run_test("y_string_comments.json")

    def test_fragment_y_string_double_escape_a(self):
        self._run_test("y_string_double_escape_a.json")

    def test_fragment_y_string_double_escape_(self):
        self._run_test("y_string_double_escape_n.json")

    def test_fragment_y_string_escaped_control_character(self):
        self._run_test("y_string_escaped_control_character.json")

    def test_fragment_y_string_escaped_noncharacter(self):
        self._run_test("y_string_escaped_noncharacter.json")

    def test_fragment_y_string_in_array(self):
        self._run_test("y_string_in_array.json")

    def test_fragment_y_string_in_array_with_leading_space(self):
        self._run_test("y_string_in_array_with_leading_space.json")

    def test_fragment_y_string_last_surrogates_1_and_2(self):
        self._run_test("y_string_last_surrogates_1_and_2.json")

    def test_fragment_y_string_nbsp_uescaped(self):
        self._run_test("y_string_nbsp_uescaped.json")

    def test_fragment_y_string_nonCharacterInUTF_8_U_10FFFF(self):
        self._run_test("y_string_nonCharacterInUTF-8_U+10FFFF.json")

    def test_fragment_y_string_nonCharacterInUTF_8_U_FFFF(self):
        self._run_test("y_string_nonCharacterInUTF-8_U+FFFF.json")

    def test_fragment_y_string_null_escape(self):
        self._run_test("y_string_null_escape.json")

    def test_fragment_y_string_one_byte_utf_8(self):
        self._run_test("y_string_one-byte-utf-8.json")

    def test_fragment_y_string_pi(self):
        self._run_test("y_string_pi.json")

    def test_fragment_y_string_reservedCharacterInUTF_8_U_1BFFF(self):
        self._run_test("y_string_reservedCharacterInUTF-8_U+1BFFF.json")

    def test_fragment_y_string_simple_ascii(self):
        self._run_test("y_string_simple_ascii.json")

    def test_fragment_y_string_space(self):
        self._run_test("y_string_space.json")

    def test_fragment_y_string_surrogates_U_1D11E_MUSICAL_SYMBOL_G_CLEF(self):
        self._run_test("y_string_surrogates_U+1D11E_MUSICAL_SYMBOL_G_CLEF.json")

    def test_fragment_y_string_three_byte_utf_8(self):
        self._run_test("y_string_three-byte-utf-8.json")

    def test_fragment_y_string_two_byte_utf_8(self):
        self._run_test("y_string_two-byte-utf-8.json")

    def test_fragment_y_string_u_2028_line_sep(self):
        self._run_test("y_string_u+2028_line_sep.json")

    def test_fragment_y_string_u_2029_par_sep(self):
        self._run_test("y_string_u+2029_par_sep.json")

    def test_fragment_y_string_uEscape(self):
        self._run_test("y_string_uEscape.json")

    def test_fragment_y_string_uescaped_newline(self):
        self._run_test("y_string_uescaped_newline.json")

    def test_fragment_y_string_unescaped_char_delete(self):
        self._run_test("y_string_unescaped_char_delete.json")

    def test_fragment_y_string_unicode(self):
        self._run_test("y_string_unicode.json")

    def test_fragment_y_string_unicodeEscapedBackslash(self):
        self._run_test("y_string_unicodeEscapedBackslash.json")

    def test_fragment_y_string_unicode_2(self):
        self._run_test("y_string_unicode_2.json")

    def test_fragment_y_string_unicode_U_10FFFE_nonchar(self):
        self._run_test("y_string_unicode_U+10FFFE_nonchar.json")

    def test_fragment_y_string_unicode_U_1FFFE_nonchar(self):
        self._run_test("y_string_unicode_U+1FFFE_nonchar.json")

    def test_fragment_y_string_unicode_U_200B_ZERO_WIDTH_SPACE(self):
        self._run_test("y_string_unicode_U+200B_ZERO_WIDTH_SPACE.json")

    def test_fragment_y_string_unicode_U_2064_invisible_plu(self):
        self._run_test("y_string_unicode_U+2064_invisible_plus.json")

    def test_fragment_y_string_unicode_U_FDD0_nonchar(self):
        self._run_test("y_string_unicode_U+FDD0_nonchar.json")

    def test_fragment_y_string_unicode_U_FFFE_nonchar(self):
        self._run_test("y_string_unicode_U+FFFE_nonchar.json")

    def test_fragment_y_string_unicode_escaped_double_quote(self):
        self._run_test("y_string_unicode_escaped_double_quote.json")

    def test_fragment_y_string_utf8(self):
        self._run_test("y_string_utf8.json")

    def test_fragment_y_string_with_del_character(self):
        self._run_test("y_string_with_del_character.json")

    def test_fragment_y_structure_lonely_false(self):
        self._run_test("y_structure_lonely_false.json")

    def test_fragment_y_structure_lonely_int(self):
        self._run_test("y_structure_lonely_int.json")

    def test_fragment_y_structure_lonely_negative_real(self):
        self._run_test("y_structure_lonely_negative_real.json")

    def test_fragment_y_structure_lonely_null(self):
        self._run_test("y_structure_lonely_null.json")

    def test_fragment_y_structure_lonely_string(self):
        self._run_test("y_structure_lonely_string.json")

    def test_fragment_y_structure_lonely_true(self):
        self._run_test("y_structure_lonely_true.json")

    def test_fragment_y_structure_string_empty(self):
        self._run_test("y_structure_string_empty.json")

    def test_fragment_y_structure_trailing_newline(self):
        self._run_test("y_structure_trailing_newline.json")

    def test_fragment_y_structure_true_in_array(self):
        self._run_test("y_structure_true_in_array.json")

    def test_fragment_y_structure_whitespace_array(self):
        self._run_test("y_structure_whitespace_array.json")

    def test_fragment_n_array_1_true_without_comma(self):
        self._run_test("n_array_1_true_without_comma.json")

    def test_fragment_n_array_a_invalid_utf8(self):
        self._run_test("n_array_a_invalid_utf8.json")

    def test_fragment_n_array_colon_instead_of_comma(self):
        self._run_test("n_array_colon_instead_of_comma.json")

    def test_fragment_n_array_comma_after_close(self):
        self._run_test("n_array_comma_after_close.json")

    def test_fragment_n_array_comma_and_number(self):
        self._run_test("n_array_comma_and_number.json")

    def test_fragment_n_array_double_comma(self):
        self._run_test("n_array_double_comma.json")

    def test_fragment_n_array_double_extra_comma(self):
        self._run_test("n_array_double_extra_comma.json")

    def test_fragment_n_array_extra_close(self):
        self._run_test("n_array_extra_close.json")

    def test_fragment_n_array_extra_comma(self):
        self._run_test("n_array_extra_comma.json")

    def test_fragment_n_array_incomplete(self):
        self._run_test("n_array_incomplete.json")

    def test_fragment_n_array_incomplete_invalid_value(self):
        self._run_test("n_array_incomplete_invalid_value.json")

    def test_fragment_n_array_inner_array_no_comma(self):
        self._run_test("n_array_inner_array_no_comma.json")

    def test_fragment_n_array_invalid_utf8(self):
        self._run_test("n_array_invalid_utf8.json")

    def test_fragment_n_array_items_separated_by_semicol(self):
        self._run_test("n_array_items_separated_by_semicolon.json")

    def test_fragment_n_array_just_comma(self):
        self._run_test("n_array_just_comma.json")

    def test_fragment_n_array_just_minu(self):
        self._run_test("n_array_just_minus.json")

    def test_fragment_n_array_missing_value(self):
        self._run_test("n_array_missing_value.json")

    def test_fragment_n_array_newlines_unclosed(self):
        self._run_test("n_array_newlines_unclosed.json")

    def test_fragment_n_array_number_and_comma(self):
        self._run_test("n_array_number_and_comma.json")

    def test_fragment_n_array_number_and_several_comma(self):
        self._run_test("n_array_number_and_several_commas.json")

    def test_fragment_n_array_spaces_vertical_tab_formfeed(self):
        self._run_test("n_array_spaces_vertical_tab_formfeed.json")

    def test_fragment_n_array_star_inside(self):
        self._run_test("n_array_star_inside.json")

    def test_fragment_n_array_unclosed(self):
        self._run_test("n_array_unclosed.json")

    def test_fragment_n_array_unclosed_trailing_comma(self):
        self._run_test("n_array_unclosed_trailing_comma.json")

    def test_fragment_n_array_unclosed_with_new_line(self):
        self._run_test("n_array_unclosed_with_new_lines.json")

    def test_fragment_n_array_unclosed_with_object_inside(self):
        self._run_test("n_array_unclosed_with_object_inside.json")

    def test_fragment_n_incomplete_false(self):
        self._run_test("n_incomplete_false.json")

    def test_fragment_n_incomplete_null(self):
        self._run_test("n_incomplete_null.json")

    def test_fragment_n_incomplete_true(self):
        self._run_test("n_incomplete_true.json")

    def test_fragment_n_multidigit_number_then_00(self):
        self._run_test("n_multidigit_number_then_00.json")

    def test_fragment_n_number__(self):
        self._run_test("n_number_++.json")

    def test_fragment_n_number_1(self):
        self._run_test("n_number_+1.json")

    def test_fragment_n_number_Inf(self):
        self._run_test("n_number_+Inf.json")

    def test_fragment_n_number_01(self):
        self._run_test("n_number_-01.json")

    def test_fragment_n_number_1_0(self):
        self._run_test("n_number_-1.0..json")

    def test_fragment_n_number_2(self):
        self._run_test("n_number_-2..json")

    def test_fragment_n_number_negative_NaN(self):
        self._run_test("n_number_-NaN.json")

    def test_fragment_n_number_negative_1(self):
        self._run_test("n_number_.-1.json")

    def test_fragment_n_number_2e_3(self):
        self._run_test("n_number_.2e-3.json")

    def test_fragment_n_number_0_1_2(self):
        self._run_test("n_number_0.1.2.json")

    def test_fragment_n_number_0_3e_(self):
        self._run_test("n_number_0.3e+.json")

    def test_fragment_n_number_0_3e(self):
        self._run_test("n_number_0.3e.json")

    def test_fragment_n_number_0_e1(self):
        self._run_test("n_number_0.e1.json")

    def test_fragment_n_number_0_capital_E_(self):
        self._run_test("n_number_0_capital_E+.json")

    def test_fragment_n_number_0_capital_E(self):
        self._run_test("n_number_0_capital_E.json")

    def test_fragment_n_number_0e_(self):
        self._run_test("n_number_0e+.json")

    def test_fragment_n_number_0e(self):
        self._run_test("n_number_0e.json")

    def test_fragment_n_number_1_0e_(self):
        self._run_test("n_number_1.0e+.json")

    def test_fragment_n_number_1_0e_2(self):
        self._run_test("n_number_1.0e-.json")

    def test_fragment_n_number_1_0e(self):
        self._run_test("n_number_1.0e.json")

    def test_fragment_n_number_1_000(self):
        self._run_test("n_number_1_000.json")

    def test_fragment_n_number_1eE2(self):
        self._run_test("n_number_1eE2.json")

    def test_fragment_n_number_2_e_3(self):
        self._run_test("n_number_2.e+3.json")

    def test_fragment_n_number_2_e_3_2(self):
        self._run_test("n_number_2.e-3.json")

    def test_fragment_n_number_2_e3_3(self):
        self._run_test("n_number_2.e3.json")

    def test_fragment_n_number_9_e_(self):
        self._run_test("n_number_9.e+.json")

    def test_fragment_n_number_negative_Inf(self):
        self._run_test("n_number_Inf.json")

    def test_fragment_n_number_NaN(self):
        self._run_test("n_number_NaN.json")

    def test_fragment_n_number_U_FF11_fullwidth_digit_one(self):
        self._run_test("n_number_U+FF11_fullwidth_digit_one.json")

    def test_fragment_n_number_expressi(self):
        self._run_test("n_number_expression.json")

    def test_fragment_n_number_hex_1_digit(self):
        self._run_test("n_number_hex_1_digit.json")

    def test_fragment_n_number_hex_2_digit(self):
        self._run_test("n_number_hex_2_digits.json")

    def test_fragment_n_number_infinity(self):
        self._run_test("n_number_infinity.json")

    def test_fragment_n_number_invalid_(self):
        self._run_test("n_number_invalid+-.json")

    def test_fragment_n_number_invalid_negative_real(self):
        self._run_test("n_number_invalid-negative-real.json")

    def test_fragment_n_number_invalid_utf_8_in_bigger_int(self):
        self._run_test("n_number_invalid-utf-8-in-bigger-int.json")

    def test_fragment_n_number_invalid_utf_8_in_exponent(self):
        self._run_test("n_number_invalid-utf-8-in-exponent.json")

    def test_fragment_n_number_invalid_utf_8_in_int(self):
        self._run_test("n_number_invalid-utf-8-in-int.json")

    def test_fragment_n_number_minus_infinity(self):
        self._run_test("n_number_minus_infinity.json")

    def test_fragment_n_number_minus_sign_with_trailing_garbage(self):
        self._run_test("n_number_minus_sign_with_trailing_garbage.json")

    def test_fragment_n_number_minus_space_1(self):
        self._run_test("n_number_minus_space_1.json")

    def test_fragment_n_number_neg_int_starting_with_zer(self):
        self._run_test("n_number_neg_int_starting_with_zero.json")

    def test_fragment_n_number_neg_real_without_int_part(self):
        self._run_test("n_number_neg_real_without_int_part.json")

    def test_fragment_n_number_neg_with_garbage_at_end(self):
        self._run_test("n_number_neg_with_garbage_at_end.json")

    def test_fragment_n_number_real_garbage_after_e(self):
        self._run_test("n_number_real_garbage_after_e.json")

    def test_fragment_n_number_real_with_invalid_utf8_after_e(self):
        self._run_test("n_number_real_with_invalid_utf8_after_e.json")

    def test_fragment_n_number_real_without_fractional_part(self):
        self._run_test("n_number_real_without_fractional_part.json")

    def test_fragment_n_number_starting_with_dot(self):
        self._run_test("n_number_starting_with_dot.json")

    def test_fragment_n_number_with_alpha(self):
        self._run_test("n_number_with_alpha.json")

    def test_fragment_n_number_with_alpha_char(self):
        self._run_test("n_number_with_alpha_char.json")

    def test_fragment_n_number_with_leading_zer(self):
        self._run_test("n_number_with_leading_zero.json")

    def test_fragment_n_object_bad_value(self):
        self._run_test("n_object_bad_value.json")

    def test_fragment_n_object_bracket_key(self):
        self._run_test("n_object_bracket_key.json")

    def test_fragment_n_object_comma_instead_of_col(self):
        self._run_test("n_object_comma_instead_of_colon.json")

    def test_fragment_n_object_double_col(self):
        self._run_test("n_object_double_colon.json")

    def test_fragment_n_object_emoji(self):
        self._run_test("n_object_emoji.json")

    def test_fragment_n_object_garbage_at_end(self):
        self._run_test("n_object_garbage_at_end.json")

    def test_fragment_n_object_key_with_single_quote(self):
        self._run_test("n_object_key_with_single_quotes.json")

    def test_fragment_n_object_lone_continuation_byte_in_key_and_trailing_comma(self):
        self._run_test("n_object_lone_continuation_byte_in_key_and_trailing_comma.json")

    def test_fragment_n_object_missing_col(self):
        self._run_test("n_object_missing_colon.json")

    def test_fragment_n_object_missing_key(self):
        self._run_test("n_object_missing_key.json")

    def test_fragment_n_object_missing_semicol(self):
        self._run_test("n_object_missing_semicolon.json")

    def test_fragment_n_object_missing_value(self):
        self._run_test("n_object_missing_value.json")

    def test_fragment_n_object_no_col(self):
        self._run_test("n_object_no-colon.json")

    def test_fragment_n_object_non_string_key(self):
        self._run_test("n_object_non_string_key.json")

    def test_fragment_n_object_non_string_key_but_huge_number_instead(self):
        self._run_test("n_object_non_string_key_but_huge_number_instead.json")

    def test_fragment_n_object_repeated_null_null(self):
        self._run_test("n_object_repeated_null_null.json")

    def test_fragment_n_object_several_trailing_comma(self):
        self._run_test("n_object_several_trailing_commas.json")

    def test_fragment_n_object_single_quote(self):
        self._run_test("n_object_single_quote.json")

    def test_fragment_n_object_trailing_comma(self):
        self._run_test("n_object_trailing_comma.json")

    def test_fragment_n_object_trailing_comment(self):
        self._run_test("n_object_trailing_comment.json")

    def test_fragment_n_object_trailing_comment_ope(self):
        self._run_test("n_object_trailing_comment_open.json")

    def test_fragment_n_object_trailing_comment_slash_ope(self):
        self._run_test("n_object_trailing_comment_slash_open.json")

    def test_fragment_n_object_trailing_comment_slash_open_incomplete(self):
        self._run_test("n_object_trailing_comment_slash_open_incomplete.json")

    def test_fragment_n_object_two_commas_in_a_row(self):
        self._run_test("n_object_two_commas_in_a_row.json")

    def test_fragment_n_object_unquoted_key(self):
        self._run_test("n_object_unquoted_key.json")

    def test_fragment_n_object_unterminated_value(self):
        self._run_test("n_object_unterminated-value.json")

    def test_fragment_n_object_with_single_string(self):
        self._run_test("n_object_with_single_string.json")

    def test_fragment_n_object_with_trailing_garbage(self):
        self._run_test("n_object_with_trailing_garbage.json")

    def test_fragment_n_single_space(self):
        self._run_test("n_single_space.json")

    def test_fragment_n_string_1_surrogate_then_escape(self):
        self._run_test("n_string_1_surrogate_then_escape.json")

    def test_fragment_n_string_1_surrogate_then_escape_u(self):
        self._run_test("n_string_1_surrogate_then_escape_u.json")

    def test_fragment_n_string_1_surrogate_then_escape_u1(self):
        self._run_test("n_string_1_surrogate_then_escape_u1.json")

    def test_fragment_n_string_1_surrogate_then_escape_u1x(self):
        self._run_test("n_string_1_surrogate_then_escape_u1x.json")

    def test_fragment_n_string_accentuated_char_no_quote(self):
        self._run_test("n_string_accentuated_char_no_quotes.json")

    def test_fragment_n_string_backslash_00(self):
        self._run_test("n_string_backslash_00.json")

    def test_fragment_n_string_escape_x(self):
        self._run_test("n_string_escape_x.json")

    def test_fragment_n_string_escaped_backslash_bad(self):
        self._run_test("n_string_escaped_backslash_bad.json")

    def test_fragment_n_string_escaped_ctrl_char_tab(self):
        self._run_test("n_string_escaped_ctrl_char_tab.json")

    def test_fragment_n_string_escaped_emoji(self):
        self._run_test("n_string_escaped_emoji.json")

    def test_fragment_n_string_incomplete_escape(self):
        self._run_test("n_string_incomplete_escape.json")

    def test_fragment_n_string_incomplete_escaped_character(self):
        self._run_test("n_string_incomplete_escaped_character.json")

    def test_fragment_n_string_incomplete_surrogate(self):
        self._run_test("n_string_incomplete_surrogate.json")

    def test_fragment_n_string_incomplete_surrogate_escape_invalid(self):
        self._run_test("n_string_incomplete_surrogate_escape_invalid.json")

    def test_fragment_n_string_invalid_utf_8_in_escape(self):
        self._run_test("n_string_invalid-utf-8-in-escape.json")

    def test_fragment_n_string_invalid_backslash_esc(self):
        self._run_test("n_string_invalid_backslash_esc.json")

    def test_fragment_n_string_invalid_unicode_escape(self):
        self._run_test("n_string_invalid_unicode_escape.json")

    def test_fragment_n_string_invalid_utf8_after_escape(self):
        self._run_test("n_string_invalid_utf8_after_escape.json")

    def test_fragment_n_string_leading_uescaped_thinspace(self):
        self._run_test("n_string_leading_uescaped_thinspace.json")

    def test_fragment_n_string_no_quotes_with_bad_escape(self):
        self._run_test("n_string_no_quotes_with_bad_escape.json")

    def test_fragment_n_string_single_doublequote(self):
        self._run_test("n_string_single_doublequote.json")

    def test_fragment_n_string_single_quote(self):
        self._run_test("n_string_single_quote.json")

    def test_fragment_n_string_single_string_no_double_quote(self):
        self._run_test("n_string_single_string_no_double_quotes.json")

    def test_fragment_n_string_start_escape_unclosed(self):
        self._run_test("n_string_start_escape_unclosed.json")

    def test_fragment_n_string_unescaped_crtl_char(self):
        self._run_test("n_string_unescaped_crtl_char.json")

    def test_fragment_n_string_unescaped_newline(self):
        self._run_test("n_string_unescaped_newline.json")

    def test_fragment_n_string_unescaped_tab(self):
        self._run_test("n_string_unescaped_tab.json")

    def test_fragment_n_string_unicode_CapitalU(self):
        self._run_test("n_string_unicode_CapitalU.json")

    def test_fragment_n_string_with_trailing_garbage(self):
        self._run_test("n_string_with_trailing_garbage.json")

    def test_fragment_n_structure_100000_opening_array(self):
        self._run_test("n_structure_100000_opening_arrays.json.xz")

    def test_fragment_n_structure_U_2060_word_joined(self):
        self._run_test("n_structure_U+2060_word_joined.json")

    def test_fragment_n_structure_UTF8_BOM_no_data(self):
        self._run_test("n_structure_UTF8_BOM_no_data.json")

    def test_fragment_n_structure_angle_bracket_(self):
        self._run_test("n_structure_angle_bracket_..json")

    def test_fragment_n_structure_angle_bracket_null(self):
        self._run_test("n_structure_angle_bracket_null.json")

    def test_fragment_n_structure_array_trailing_garbage(self):
        self._run_test("n_structure_array_trailing_garbage.json")

    def test_fragment_n_structure_array_with_extra_array_close(self):
        self._run_test("n_structure_array_with_extra_array_close.json")

    def test_fragment_n_structure_array_with_unclosed_string(self):
        self._run_test("n_structure_array_with_unclosed_string.json")

    def test_fragment_n_structure_ascii_unicode_identifier(self):
        self._run_test("n_structure_ascii-unicode-identifier.json")

    def test_fragment_n_structure_capitalized_True(self):
        self._run_test("n_structure_capitalized_True.json")

    def test_fragment_n_structure_close_unopened_array(self):
        self._run_test("n_structure_close_unopened_array.json")

    def test_fragment_n_structure_comma_instead_of_closing_brace(self):
        self._run_test("n_structure_comma_instead_of_closing_brace.json")

    def test_fragment_n_structure_double_array(self):
        self._run_test("n_structure_double_array.json")

    def test_fragment_n_structure_end_array(self):
        self._run_test("n_structure_end_array.json")

    def test_fragment_n_structure_incomplete_UTF8_BOM(self):
        self._run_test("n_structure_incomplete_UTF8_BOM.json")

    def test_fragment_n_structure_lone_invalid_utf_8(self):
        self._run_test("n_structure_lone-invalid-utf-8.json")

    def test_fragment_n_structure_lone_open_bracket(self):
        self._run_test("n_structure_lone-open-bracket.json")

    def test_fragment_n_structure_no_data(self):
        self._run_test("n_structure_no_data.json")

    def test_fragment_n_structure_null_byte_outside_string(self):
        self._run_test("n_structure_null-byte-outside-string.json")

    def test_fragment_n_structure_number_with_trailing_garbage(self):
        self._run_test("n_structure_number_with_trailing_garbage.json")

    def test_fragment_n_structure_object_followed_by_closing_object(self):
        self._run_test("n_structure_object_followed_by_closing_object.json")

    def test_fragment_n_structure_object_unclosed_no_value(self):
        self._run_test("n_structure_object_unclosed_no_value.json")

    def test_fragment_n_structure_object_with_comment(self):
        self._run_test("n_structure_object_with_comment.json")

    def test_fragment_n_structure_object_with_trailing_garbage(self):
        self._run_test("n_structure_object_with_trailing_garbage.json")

    def test_fragment_n_structure_open_array_apostrophe(self):
        self._run_test("n_structure_open_array_apostrophe.json")

    def test_fragment_n_structure_open_array_comma(self):
        self._run_test("n_structure_open_array_comma.json")

    def test_fragment_n_structure_open_array_object(self):
        self._run_test("n_structure_open_array_object.json.xz")

    def test_fragment_n_structure_open_array_open_object(self):
        self._run_test("n_structure_open_array_open_object.json")

    def test_fragment_n_structure_open_array_open_string(self):
        self._run_test("n_structure_open_array_open_string.json")

    def test_fragment_n_structure_open_array_string(self):
        self._run_test("n_structure_open_array_string.json")

    def test_fragment_n_structure_open_object(self):
        self._run_test("n_structure_open_object.json")

    def test_fragment_n_structure_open_object_close_array(self):
        self._run_test("n_structure_open_object_close_array.json")

    def test_fragment_n_structure_open_object_comma(self):
        self._run_test("n_structure_open_object_comma.json")

    def test_fragment_n_structure_open_object_open_array(self):
        self._run_test("n_structure_open_object_open_array.json")

    def test_fragment_n_structure_open_object_open_string(self):
        self._run_test("n_structure_open_object_open_string.json")

    def test_fragment_n_structure_open_object_string_with_apostrophe(self):
        self._run_test("n_structure_open_object_string_with_apostrophes.json")

    def test_fragment_n_structure_open_ope(self):
        self._run_test("n_structure_open_open.json")

    def test_fragment_n_structure_single_eacute(self):
        self._run_test("n_structure_single_eacute.json")

    def test_fragment_n_structure_single_star(self):
        self._run_test("n_structure_single_star.json")

    def test_fragment_n_structure_trailing_(self):
        self._run_test("n_structure_trailing_#.json")

    def test_fragment_n_structure_uescaped_LF_before_string(self):
        self._run_test("n_structure_uescaped_LF_before_string.json")

    def test_fragment_n_structure_unclosed_array(self):
        self._run_test("n_structure_unclosed_array.json")

    def test_fragment_n_structure_unclosed_array_partial_null(self):
        self._run_test("n_structure_unclosed_array_partial_null.json")

    def test_fragment_n_structure_unclosed_array_unfinished_false(self):
        self._run_test("n_structure_unclosed_array_unfinished_false.json")

    def test_fragment_n_structure_unclosed_array_unfinished_true(self):
        self._run_test("n_structure_unclosed_array_unfinished_true.json")

    def test_fragment_n_structure_unclosed_object(self):
        self._run_test("n_structure_unclosed_object.json")

    def test_fragment_n_structure_unicode_identifier(self):
        self._run_test("n_structure_unicode-identifier.json")

    def test_fragment_n_structure_whitespace_U_2060_word_joiner(self):
        self._run_test("n_structure_whitespace_U+2060_word_joiner.json")

    def test_fragment_n_structure_whitespace_formfeed(self):
        self._run_test("n_structure_whitespace_formfeed.json")

    def test_fragment_i_number_double_huge_neg_exp(self):
        self._run_test("i_number_double_huge_neg_exp.json")

    def test_fragment_i_number_huge_exp(self):
        self._run_test("i_number_huge_exp.json")

    def test_fragment_i_number_neg_int_huge_exp(self):
        self._run_test("i_number_neg_int_huge_exp.json")

    def test_fragment_i_number_pos_double_huge_exp(self):
        self._run_test("i_number_pos_double_huge_exp.json")

    def test_fragment_i_number_real_neg_overflow(self):
        self._run_test("i_number_real_neg_overflow.json")

    def test_fragment_i_number_real_pos_overflow(self):
        self._run_test("i_number_real_pos_overflow.json")

    def test_fragment_i_number_real_underflow(self):
        self._run_test("i_number_real_underflow.json")

    def test_fragment_i_number_too_big_neg_int(self):
        self._run_test("i_number_too_big_neg_int.json")

    def test_fragment_i_number_too_big_pos_int(self):
        self._run_test("i_number_too_big_pos_int.json")

    def test_fragment_i_number_very_big_negative_int(self):
        self._run_test("i_number_very_big_negative_int.json")

    def test_fragment_i_object_key_lone_2nd_surrogate(self):
        self._run_test("i_object_key_lone_2nd_surrogate.json")

    def test_fragment_i_string_1st_surrogate_but_2nd_missing(self):
        self._run_test("i_string_1st_surrogate_but_2nd_missing.json")

    def test_fragment_i_string_1st_valid_surrogate_2nd_invalid(self):
        self._run_test("i_string_1st_valid_surrogate_2nd_invalid.json")

    def test_fragment_i_string_UTF_16LE_with_BOM(self):
        self._run_test("i_string_UTF-16LE_with_BOM.json")

    def test_fragment_i_string_UTF_8_invalid_sequence(self):
        self._run_test("i_string_UTF-8_invalid_sequence.json")

    def test_fragment_i_string_UTF8_surrogate_U_D800(self):
        self._run_test("i_string_UTF8_surrogate_U+D800.json")

    def test_fragment_i_string_incomplete_surrogate_and_escape_valid(self):
        self._run_test("i_string_incomplete_surrogate_and_escape_valid.json")

    def test_fragment_i_string_incomplete_surrogate_pair(self):
        self._run_test("i_string_incomplete_surrogate_pair.json")

    def test_fragment_i_string_incomplete_surrogates_escape_valid(self):
        self._run_test("i_string_incomplete_surrogates_escape_valid.json")

    def test_fragment_i_string_invalid_lonely_surrogate(self):
        self._run_test("i_string_invalid_lonely_surrogate.json")

    def test_fragment_i_string_invalid_surrogate(self):
        self._run_test("i_string_invalid_surrogate.json")

    def test_fragment_i_string_invalid_utf_8(self):
        self._run_test("i_string_invalid_utf-8.json")

    def test_fragment_i_string_inverted_surrogates_U_1D11E(self):
        self._run_test("i_string_inverted_surrogates_U+1D11E.json")

    def test_fragment_i_string_iso_latin_1(self):
        self._run_test("i_string_iso_latin_1.json")

    def test_fragment_i_string_lone_second_surrogate(self):
        self._run_test("i_string_lone_second_surrogate.json")

    def test_fragment_i_string_lone_utf8_continuation_byte(self):
        self._run_test("i_string_lone_utf8_continuation_byte.json")

    def test_fragment_i_string_not_in_unicode_range(self):
        self._run_test("i_string_not_in_unicode_range.json")

    def test_fragment_i_string_overlong_sequence_2_byte(self):
        self._run_test("i_string_overlong_sequence_2_bytes.json")

    def test_fragment_i_string_overlong_sequence_6_byte(self):
        self._run_test("i_string_overlong_sequence_6_bytes.json")

    def test_fragment_i_string_overlong_sequence_6_bytes_null(self):
        self._run_test("i_string_overlong_sequence_6_bytes_null.json")

    def test_fragment_i_string_truncated_utf_8(self):
        self._run_test("i_string_truncated-utf-8.json")

    def test_fragment_i_string_utf16BE_no_BOM(self):
        self._run_test("i_string_utf16BE_no_BOM.json")

    def test_fragment_i_string_utf16LE_no_BOM(self):
        self._run_test("i_string_utf16LE_no_BOM.json")

    def test_fragment_i_structure_500_nested_array(self):
        self._run_test("i_structure_500_nested_arrays.json.xz")

    def test_fragment_i_structure_UTF_8_BOM_empty_object(self):
        self._run_test("i_structure_UTF-8_BOM_empty_object.json")



# SPDX-License-Identifier: (Apache-2.0 OR MIT)


import json

import pytest

import orjson


class TestUltraJSON:
    def test_doubleLongIssue(self):
        sut = {"a": -4342969734183514}
        encoded = orjson.dumps(sut)
        decoded = orjson.loads(encoded)
        assert sut == decoded
        encoded = orjson.dumps(sut)
        decoded = orjson.loads(encoded)
        assert sut == decoded

    def test_doubleLongDecimalIssue(self):
        sut = {"a": -12345678901234.56789012}
        encoded = orjson.dumps(sut)
        decoded = orjson.loads(encoded)
        assert sut == decoded
        encoded = orjson.dumps(sut)
        decoded = orjson.loads(encoded)
        assert sut == decoded

    def test_encodeDecodeLongDecimal(self):
        sut = {"a": -528656961.4399388}
        encoded = orjson.dumps(sut)
        orjson.loads(encoded)

    def test_decimalDecodeTest(self):
        sut = {"a": 4.56}
        encoded = orjson.dumps(sut)
        decoded = orjson.loads(encoded)
        pytest.approx(sut["a"], decoded["a"])

    def test_encodeDictWithUnicodeKeys(self):
        val = {
            "key1": "value1",
            "key1": "value1",
            "key1": "value1",
            "key1": "value1",
            "key1": "value1",
            "key1": "value1",
        }
        orjson.dumps(val)

        val = {
            "بن": "value1",
            "بن": "value1",
            "بن": "value1",
            "بن": "value1",
            "بن": "value1",
            "بن": "value1",
            "بن": "value1",
        }
        orjson.dumps(val)

    def test_encodeArrayOfNestedArrays(self):
        val = [[[[]]]] * 20  # type: ignore
        output = orjson.dumps(val)
        assert val == orjson.loads(output)
        assert val == orjson.loads(output)

    def test_encodeArrayOfDoubles(self):
        val = [31337.31337, 31337.31337, 31337.31337, 31337.31337] * 10
        output = orjson.dumps(val)
        assert val == orjson.loads(output)
        assert val == orjson.loads(output)

    def test_encodeStringConversion2(self):
        val = "A string \\ / \b \f \n \r \t"
        output = orjson.dumps(val)
        assert val == orjson.loads(output)
        assert output == b'"A string \\\\ / \\b \\f \\n \\r \\t"'
        assert val == orjson.loads(output)

    def test_decodeUnicodeConversion(self):
        pass

    def test_encodeUnicodeConversion1(self):
        val = "Räksmörgås اسامة بن محمد بن عوض بن لادن"
        enc = orjson.dumps(val)
        dec = orjson.loads(enc)
        assert enc == orjson.dumps(val)
        assert dec == orjson.loads(enc)

    def test_encodeControlEscaping(self):
        val = "\x19"
        enc = orjson.dumps(val)
        dec = orjson.loads(enc)
        assert val == dec
        assert enc == orjson.dumps(val)

    def test_encodeUnicodeConversion2(self):
        val = "\xe6\x97\xa5\xd1\x88"
        enc = orjson.dumps(val)
        dec = orjson.loads(enc)
        assert enc == orjson.dumps(val)
        assert dec == orjson.loads(enc)

    def test_encodeUnicodeSurrogatePair(self):
        val = "\xf0\x90\x8d\x86"
        enc = orjson.dumps(val)
        dec = orjson.loads(enc)

        assert enc == orjson.dumps(val)
        assert dec == orjson.loads(enc)

    def test_encodeUnicode4BytesUTF8(self):
        val = "\xf0\x91\x80\xb0TRAILINGNORMAL"
        enc = orjson.dumps(val)
        dec = orjson.loads(enc)

        assert enc == orjson.dumps(val)
        assert dec == orjson.loads(enc)

    def test_encodeUnicode4BytesUTF8Highest(self):
        val = "\xf3\xbf\xbf\xbfTRAILINGNORMAL"
        enc = orjson.dumps(val)
        dec = orjson.loads(enc)

        assert enc == orjson.dumps(val)
        assert dec == orjson.loads(enc)

    def testEncodeUnicodeBMP(self):
        s = "\U0001f42e\U0001f42e\U0001f42d\U0001f42d"  # 🐮🐮🐭🐭
        orjson.dumps(s)
        json.dumps(s)

        assert json.loads(json.dumps(s)) == s
        assert orjson.loads(orjson.dumps(s)) == s

    def testEncodeSymbols(self):
        s = "\u273f\u2661\u273f"  # ✿♡✿
        encoded = orjson.dumps(s)
        encoded_json = json.dumps(s)

        decoded = orjson.loads(encoded)
        assert s == decoded

        encoded = orjson.dumps(s)

        # json outputs an unicode object
        encoded_json = json.dumps(s, ensure_ascii=False)
        assert encoded == encoded_json.encode("utf-8")
        decoded = orjson.loads(encoded)
        assert s == decoded

    def test_encodeArrayInArray(self):
        val = [[[[]]]]  # type: ignore
        output = orjson.dumps(val)

        assert val == orjson.loads(output)
        assert output == orjson.dumps(val)
        assert val == orjson.loads(output)

    def test_encodeIntConversion(self):
        val = 31337
        output = orjson.dumps(val)
        assert val == orjson.loads(output)
        assert output == orjson.dumps(val)
        assert val == orjson.loads(output)

    def test_encodeIntNegConversion(self):
        val = -31337
        output = orjson.dumps(val)
        assert val == orjson.loads(output)
        assert output == orjson.dumps(val)
        assert val == orjson.loads(output)

    def test_encodeLongNegConversion(self):
        val = -9223372036854775808
        output = orjson.dumps(val)

        orjson.loads(output)
        orjson.loads(output)

        assert val == orjson.loads(output)
        assert output == orjson.dumps(val)
        assert val == orjson.loads(output)

    def test_encodeListConversion(self):
        val = [1, 2, 3, 4]
        output = orjson.dumps(val)
        assert val == orjson.loads(output)
        assert val == orjson.loads(output)

    def test_encodeDictConversion(self):
        val = {"k1": 1, "k2": 2, "k3": 3, "k4": 4}
        output = orjson.dumps(val)
        assert val == orjson.loads(output)
        assert val == orjson.loads(output)
        assert val == orjson.loads(output)

    def test_encodeNoneConversion(self):
        val = None
        output = orjson.dumps(val)
        assert val == orjson.loads(output)
        assert output == orjson.dumps(val)
        assert val == orjson.loads(output)

    def test_encodeTrueConversion(self):
        val = True
        output = orjson.dumps(val)
        assert val == orjson.loads(output)
        assert output == orjson.dumps(val)
        assert val == orjson.loads(output)

    def test_encodeFalseConversion(self):
        val = False
        output = orjson.dumps(val)
        assert val == orjson.loads(output)
        assert output == orjson.dumps(val)
        assert val == orjson.loads(output)

    def test_encodeToUTF8(self):
        val = b"\xe6\x97\xa5\xd1\x88".decode("utf-8")
        enc = orjson.dumps(val)
        dec = orjson.loads(enc)
        assert enc == orjson.dumps(val)
        assert dec == orjson.loads(enc)

    def test_decodeFromUnicode(self):
        val = '{"obj": 31337}'
        dec1 = orjson.loads(val)
        dec2 = orjson.loads(str(val))
        assert dec1 == dec2

    def test_decodeJibberish(self):
        val = "fdsa sda v9sa fdsa"
        pytest.raises(orjson.JSONDecodeError, orjson.loads, val)

    def test_decodeBrokenArrayStart(self):
        val = "["
        pytest.raises(orjson.JSONDecodeError, orjson.loads, val)

    def test_decodeBrokenObjectStart(self):
        val = "{"
        pytest.raises(orjson.JSONDecodeError, orjson.loads, val)

    def test_decodeBrokenArrayEnd(self):
        val = "]"
        pytest.raises(orjson.JSONDecodeError, orjson.loads, val)

    def test_decodeBrokenObjectEnd(self):
        val = "}"
        pytest.raises(orjson.JSONDecodeError, orjson.loads, val)

    def test_decodeObjectDepthTooBig(self):
        val = "{" * (1024 * 1024)
        pytest.raises(orjson.JSONDecodeError, orjson.loads, val)

    def test_decodeStringUnterminated(self):
        val = '"TESTING'
        pytest.raises(orjson.JSONDecodeError, orjson.loads, val)

    def test_decodeStringUntermEscapeSequence(self):
        val = '"TESTING\\"'
        pytest.raises(orjson.JSONDecodeError, orjson.loads, val)

    def test_decodeStringBadEscape(self):
        val = '"TESTING\\"'
        pytest.raises(orjson.JSONDecodeError, orjson.loads, val)

    def test_decodeTrueBroken(self):
        val = "tru"
        pytest.raises(orjson.JSONDecodeError, orjson.loads, val)

    def test_decodeFalseBroken(self):
        val = "fa"
        pytest.raises(orjson.JSONDecodeError, orjson.loads, val)

    def test_decodeNullBroken(self):
        val = "n"
        pytest.raises(orjson.JSONDecodeError, orjson.loads, val)

    def test_decodeBrokenDictKeyTypeLeakTest(self):
        val = '{{1337:""}}'
        for _ in range(1000):
            pytest.raises(orjson.JSONDecodeError, orjson.loads, val)

    def test_decodeBrokenDictLeakTest(self):
        val = '{{"key":"}'
        for _ in range(1000):
            pytest.raises(orjson.JSONDecodeError, orjson.loads, val)

    def test_decodeBrokenListLeakTest(self):
        val = "[[[true"
        for _ in range(1000):
            pytest.raises(orjson.JSONDecodeError, orjson.loads, val)

    def test_decodeDictWithNoKey(self):
        val = "{{{{31337}}}}"
        pytest.raises(orjson.JSONDecodeError, orjson.loads, val)

    def test_decodeDictWithNoColonOrValue(self):
        val = '{{{{"key"}}}}'
        pytest.raises(orjson.JSONDecodeError, orjson.loads, val)

    def test_decodeDictWithNoValue(self):
        val = '{{{{"key":}}}}'
        pytest.raises(orjson.JSONDecodeError, orjson.loads, val)

    def test_decodeNumericIntPos(self):
        val = "31337"
        assert 31337 == orjson.loads(val)

    def test_decodeNumericIntNeg(self):
        assert -31337 == orjson.loads("-31337")

    def test_encodeNullCharacter(self):
        val = "31337 \x00 1337"
        output = orjson.dumps(val)
        assert val == orjson.loads(output)
        assert output == orjson.dumps(val)
        assert val == orjson.loads(output)

        val = "\x00"
        output = orjson.dumps(val)
        assert val == orjson.loads(output)
        assert output == orjson.dumps(val)
        assert val == orjson.loads(output)

        assert b'"  \\u0000\\r\\n "' == orjson.dumps("  \u0000\r\n ")

    def test_decodeNullCharacter(self):
        val = '"31337 \\u0000 31337"'
        assert orjson.loads(val) == json.loads(val)

    def test_decodeEscape(self):
        base = "\u00e5".encode()
        quote = b'"'
        val = quote + base + quote
        assert json.loads(val) == orjson.loads(val)

    def test_decodeBigEscape(self):
        for _ in range(10):
            base = "\u00e5".encode()
            quote = b'"'
            val = quote + (base * 1024 * 1024 * 2) + quote
            assert json.loads(val) == orjson.loads(val)



# SPDX-License-Identifier: (Apache-2.0 OR MIT)

from json import loads as json_loads

import pytest

from .data import fixtures, libraries
from .util import read_fixture_obj


@pytest.mark.parametrize("library", libraries)
@pytest.mark.parametrize("fixture", fixtures)
def test_dumps(benchmark, fixture, library):
    dumper, loader = libraries[library]
    benchmark.group = f"{fixture} serialization"
    benchmark.extra_info["lib"] = library
    data = read_fixture_obj(f"{fixture}.xz")
    benchmark.extra_info["correct"] = json_loads(dumper(data)) == data  # type: ignore
    benchmark(dumper, data)
